@@ -20,6 +20,7 @@ Sample code:
     message = mailer.Message()
     message.From = "me@example.com"
     message.To = "you@example.com"
+    message.RTo = "you@example.com"
     message.Subject = "My Vacation"
     message.Body = open("letter.txt", "rb").read()
     message.attach("picture.jpg")
@@ -28,7 +29,9 @@ Sample code:
     sender.send(message)
 
 """
+from __future__ import with_statement
 import smtplib
+import socket
 import threading
 import Queue
 import uuid
@@ -38,7 +41,6 @@ import uuid
 try:
     from email import encoders
     from email.header import make_header
-    from email.message import Message
     from email.mime.audio import MIMEAudio
     from email.mime.base import MIMEBase
     from email.mime.image import MIMEImage
@@ -47,7 +49,6 @@ try:
 except ImportError:
     from email import Encoders as encoders
     from email.Header import make_header
-    from email.MIMEMessage import Message
     from email.MIMEAudio import MIMEAudio
     from email.MIMEBase import MIMEBase
     from email.MIMEImage import MIMEImage
@@ -60,11 +61,12 @@ import time
 
 from os import path
 
-__version__ = "0.7"
+__version__ = "0.8.1"
 __author__ = "Ryan Ginstrom"
 __license__ = "MIT"
 __description__ = "A module to send email simply in Python"
 
+
 class Mailer(object):
     """
     Represents an SMTP connection.
@@ -72,18 +74,22 @@ class Mailer(object):
     Use login() to log in with a username and password.
     """
 
-    def __init__(self, host="localhost", port=0, use_tls=False, usr=None, pwd=None):
-        self.host = host
-        self.port = port
-        self.use_tls = use_tls
-        self._usr = usr
-        self._pwd = pwd
+    def __init__(self, host="localhost", port=0, use_tls=False, usr=None, pwd=None, use_ssl=False, use_plain_auth=False,
+                 timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
+        self.host           = host
+        self.port           = port
+        self.use_tls        = use_tls
+        self.use_ssl        = use_ssl
+        self.use_plain_auth = use_plain_auth
+        self._usr           = usr
+        self._pwd           = pwd
+        self.timeout        = timeout
 
     def login(self, usr, pwd):
         self._usr = usr
         self._pwd = pwd
 
-    def send(self, msg):
+    def send(self, msg, debug=False):
         """
         Send one message or a sequence of messages.
 
@@ -92,7 +98,13 @@ class Mailer(object):
         them as a list:
         mailer.send([msg1, msg2, msg3])
         """
-        server = smtplib.SMTP(self.host, self.port)
+        if self.use_ssl:
+            server = smtplib.SMTP_SSL(self.host, self.port, timeout=self.timeout)
+        else:
+            server = smtplib.SMTP(self.host, self.port, timeout=self.timeout)
+
+        if debug:
+            server.set_debuglevel(1)
 
         if self._usr and self._pwd:
             if self.use_tls is True:
@@ -100,14 +112,16 @@ class Mailer(object):
                 server.starttls()
                 server.ehlo()
 
+            if self.use_plain_auth is True:
+                server.esmtp_features["auth"] = "LOGIN PLAIN"
+
             server.login(self._usr, self._pwd)
 
-        try:
-            num_msgs = len(msg)
-            for m in msg:
-                self._send(server, m)
-        except TypeError:
-            self._send(server, msg)
+        if isinstance(msg, Message):
+            msg = [msg]
+
+        for m in msg:
+            self._send(server, m)
 
         server.quit()
 
@@ -136,14 +150,22 @@ class Mailer(object):
             else:
                 bcc = list(msg.BCC)
 
+        rto = []
+        if msg.RTo:
+            if isinstance(msg.RTo, basestring):
+                rto = [msg.RTo]
+            else:
+                rto = list(msg.RTo)
+
         you = to + cc + bcc
         server.sendmail(me, you, msg.as_string())
 
+
 class Message(object):
     """
     Represents an email message.
 
-    Set the To, From, Subject, and Body attributes as plain-text strings.
+    Set the To, From, Reply-To, Subject, and Body attributes as plain-text strings.
     Optionally, set the Html attribute to send an HTML email, or use the
     attach() method to attach files.
 
@@ -158,37 +180,61 @@ class Message(object):
     Send using the Mailer class.
     """
 
-    def __init__(self, To=None, From=None, CC=None, BCC=None, Subject=None, Body=None, Html=None,
-                 Date=None, attachments=None, charset=None):
+    def __init__(self, **kwargs):
+        """
+        Parameters and default values (parameter names are case insensitive):
+            To=None, From=None, RTo=None, CC=None, BCC=None, Subject=None, Body=None, Html=None,
+            Date=None, Attachments=None, Charset=None, Headers=None
+        """
+
+        # extract  parameters and convert names to lowercase
+        params = {}
+        for i in kwargs:
+            params[i.lower()] = kwargs[i]
+
+        # preprocess attachments
         self.attachments = []
+        attachments = params.get('attachments', None)
         if attachments:
             for attachment in attachments:
                 if isinstance(attachment, basestring):
-                    self.attachments.append((attachment, None, None))
+                    self.attachments.append((attachment, None, None, None, None))
                 else:
                     try:
-                        filename, cid = attachment
-                    except (TypeError, IndexError):
-                        self.attachments.append((attachment, None, None))
+                        length = len(attachment)
+                    except TypeError:
+                        length = None
                     else:
-                        self.attachments.append((filename, cid, None))
-        self.To = To
-        self.CC = CC
-        self.BCC = BCC
-        """string or iterable"""
-        self.From = From
-        """string"""
-        self.Subject = Subject
-        self.Body = Body
-        self.Html = Html
-        self.Date = Date or time.strftime("%a, %d %b %Y %H:%M:%S %z", time.gmtime())
-        self.charset = charset or 'us-ascii'
+                        if length is None or length <= 4:
+                            self.attachments.append((attachment, None, None, None, None))
+                        else:
+                            self.attachments.append((tuple(attachment) + (None, None, None, None))[:4])
+
+
+        self.To         = params.get('to', None)
+        self.RTo        = params.get('rto', None)
+        self.CC         = params.get('cc', None)
+        self.BCC        = params.get('bcc', None)
+        self.From       = params.get('from', None) # string or iterable
+        self.Subject    = params.get('subject', u'') # string
+        self.Body       = params.get('body', None)
+        self.Html       = params.get('html', None)
+        self.Date       = params.get('date', time.strftime("%a, %d %b %Y %H:%M:%S %z", time.gmtime()))
+        self.charset    = params.get('charset', 'us-ascii')
+        self.Headers    = params.get('headers', {})
+
+        if isinstance(self.Body, unicode):
+            self.Body = self.Body.encode(self.charset)
 
         self.message_id = self.make_key()
 
+
     def make_key(self):
         return str(uuid.uuid4())
 
+    def header(self, key, value):
+        self.Headers[key] = value
+
     def as_string(self):
         """Get the email as a string to send in the mailer"""
 
@@ -203,7 +249,7 @@ class Message(object):
         if not self.Html:
             msg = MIMEText(self.Body, 'plain', self.charset)
         else:
-            msg  = self._with_html()
+            msg = self._with_html()
 
         self._set_info(msg)
         return msg.as_string()
@@ -224,11 +270,21 @@ class Message(object):
     def _set_info(self, msg):
         if self.charset == 'us-ascii':
             msg['Subject'] = self.Subject
+            msg['From'] = self.From
+
         else:
-            subject = unicode(self.Subject, self.charset)
+            if isinstance(self.Subject, unicode):
+                subject = self.Subject
+            else:
+                subject = unicode(self.Subject, self.charset)
             msg['Subject'] = str(make_header([(subject, self.charset)]))
 
-        msg['From'] = self.From
+            if isinstance(self.From, unicode):
+                from_ = self.From
+            else:
+                from_ = unicode(self.From, self.charset)
+            msg['From'] = str(make_header([(from_, self.charset)]))
+
 
         if isinstance(self.To, basestring):
             msg['To'] = self.To
@@ -236,6 +292,13 @@ class Message(object):
             self.To = list(self.To)
             msg['To'] = ", ".join(self.To)
 
+        if self.RTo:
+            if isinstance(self.RTo, basestring):
+                msg.add_header('reply-to', self.RTo)
+            else:
+                self.RTo = list(self.RTo)
+                msg.add_header('reply-to', ", ".join(self.RTo))
+
         if self.CC:
             if isinstance(self.CC, basestring):
                 msg['CC'] = self.CC
@@ -243,6 +306,20 @@ class Message(object):
                 self.CC = list(self.CC)
                 msg['CC'] = ", ".join(self.CC)
 
+
+        if self.BCC:
+            if isinstance(self.BCC, basestring):
+                msg['BCC'] = self.BCC
+            else:
+                self.BCC = list(self.BCC)
+                msg['BCC'] = ", ".join(self.BCC)
+
+
+        if self.Headers:
+            for key, value in self.Headers.items():
+                msg[key] = str(value).encode(self.charset)
+
+
         msg['Date'] = self.Date
 
     def _multipart(self):
@@ -268,12 +345,12 @@ class Message(object):
         self._set_info(msg)
         msg.preamble = self.Subject
 
-        for filename, cid, mimetype in self.attachments:
-            self._add_attachment(msg, filename, cid, mimetype)
+        for filename, cid, mimetype, content, charset in self.attachments:
+            self._add_attachment(msg, filename, cid, mimetype, content, charset)
 
         return msg.as_string()
 
-    def _add_attachment(self, outer, filename, cid, mimetype):
+    def _add_attachment(self, outer, filename, cid, mimetype, content, charset):
         """
         If mimetype is None, it will try to guess the mimetype
         """
@@ -282,25 +359,29 @@ class Message(object):
             encoding = None
         else:
             ctype, encoding = mimetypes.guess_type(filename)
+
         if ctype is None or encoding is not None:
             # No guess could be made, or the file is encoded (compressed), so
             # use a generic bag-of-bits type.
             ctype = 'application/octet-stream'
+
         maintype, subtype = ctype.split('/', 1)
-        fp = open(filename, 'rb')
+        if not content:
+            with open(filename, 'rb') as fp:
+                content = fp.read()
+
         if maintype == 'text':
             # Note: we should handle calculating the charset
-            msg = MIMEText(fp.read(), _subtype=subtype)
+            msg = MIMEText(content, _subtype=subtype, _charset=charset)
         elif maintype == 'image':
-            msg = MIMEImage(fp.read(), _subtype=subtype)
+            msg = MIMEImage(content, _subtype=subtype)
         elif maintype == 'audio':
-            msg = MIMEAudio(fp.read(), _subtype=subtype)
+            msg = MIMEAudio(content, _subtype=subtype)
         else:
             msg = MIMEBase(maintype, subtype)
-            msg.set_payload(fp.read())
+            msg.set_payload(content)
             # Encode the payload using Base64
             encoders.encode_base64(msg)
-        fp.close()
 
         # Set the content-ID header
         if cid:
@@ -309,28 +390,31 @@ class Message(object):
         else:
             # Set the filename parameter
             msg.add_header('Content-Disposition', 'attachment', filename=path.basename(filename))
+
         outer.attach(msg)
 
-    def attach(self, filename, cid=None, mimetype=None):
+    def attach(self, filename, cid=None, mimetype=None, content=None, charset=None):
         """
         Attach a file to the email. Specify the name of the file;
         Message will figure out the MIME type and load the file.
 
-        Specify mimetype to set the MIME type manually.
+        Specify mimetype to set the MIME type manually. The content
+        argument take the contents of the file if they are already loaded
+        in memory.
         """
 
-        self.attachments.append((filename, cid, mimetype))
+        self.attachments.append((filename, cid, mimetype, content, charset))
 
 
 class Manager(threading.Thread):
     """
-    Manages the sending of email in the background
+    Manages the sending of email in the background.
 
-    you can supply it with an instance of class Mailler or pass in the same
-    parameters that you would have used to create an instance of Mailler
+    You can supply it with an instance of class Mailer or pass in the same
+    parameters that you would have used to create an instance of Mailer.
 
-    if a message was succesfully sent, self.results[msg.message_id] returns a 3
-    element tuple (True/False, err_code, err_message)
+    If a message was succesfully sent, self.results[msg.message_id] returns a 3
+    element tuple (True/False, err_code, err_message).
     """
 
     def __init__(self, mailer=None, callback=None, **kwargs):
@@ -366,10 +450,7 @@ class Manager(threading.Thread):
             if msg is None:
                 break
 
-            try:
-                num_msgs = len(msg)
-            except TypeError:
-                num_msgs = 1
+            if isinstance(msg, Message):
                 msg = [msg]
 
             for m in msg:
@@ -388,7 +469,7 @@ class Manager(threading.Thread):
                 if self.callback:
                     try:
                         self.callback(m.message_id)
-                    except:
+                    except Exception:
                         pass
 
             # endfor
